=== added directory 'cep'
=== added file 'cep/CEP-1.txt'
--- cep/CEP-1.txt	1970-01-01 00:00:00 +0000
+++ cep/CEP-1.txt	2014-12-19 12:11:52 +0000
@@ -0,0 +1,122 @@
+Checkbox Enhancement Proposal 1: Template Jobs
+
+Summary
+=======
+
+I'd like to propose that we design and implement template jobs and replace all
+occurrences of local jobs with equivalent template jobs.
+
+Rationale
+=========
+
+Local jobs are very problematic for applications and the core alike. They
+prevent us from having access to a complete set of jobs all times. That, in
+turn, prevent us from having simple application execution semantics and
+severely limits our possibility of catching bugs in job definitions.
+
+The mechanism of template jobs is currently used to generate new jobs at
+runtime, typically to do one test for each of some device or other entity). The
+code that generates such values is very arcane and goes back deep into old
+checkbox support scripts. Such essential task should be handled by the library
+as first-class object, with proper definition, semantics and implementation
+quality.
+
+As a side feature, local jobs are abused to generate "categories". This
+proposal does not touch that aspect, except in "future work" section below 
+
+Overview of template jobs
+=========================
+
+Template jobs are just like local jobs in expressiveness, with the limitation
+that the set of generated jobs (defined as a collection of their attributes,
+including their name) are constrained and known statically without executing
+any code provided by the job definition.
+
+Instead, of parsing output of arbitrary shell commands, the job definition is
+given explicitly. As a new feature, each attribute of a template job can be
+parametrized. The list of parameters is known statically (cannot be generated).
+This allows us to enumerate possible patterns and instantiate jobs when
+necessary.
+
+Each parameter is bound to an attribute of a resource. This creates a
+dependency on a specific resource job from any local job. In addition, to
+implement a common theme used by local jobs, the set of resource records can be
+filtered by a restricted python expression. This is similar in spirit to
+existing requirements system.
+
+For this proposal, each template job must use exactly one resource. In future
+this requirement may be lifted, if required.
+
+Example
+=======
+
+The following example converts an existing local job, optical/read, to a template job. This is the
+(abbreviated) text of the original definition.
+
+    plugin: local
+    name: optical/read
+    requires:
+     device.category == 'CDROM'
+    _description: Optical read test.
+    command:
+     cat <<'EOF' | run_templates -t -s 'udev_resource | filter_templates -w "category=CDROM"'
+     plugin: user-interact-verify
+     name: optical/read_`ls /sys$path/block`
+     requires: device.path == "$path"
+     user: root
+     command: optical_read_test /dev/`ls /sys$path/block`
+     description: (removed)
+     EOF
+
+This is the (equivalent template job). The actual "local" job fragment of the
+previous text is now gone. Any attribute may now use python formatting
+language. This language is safe and can be parsed and analyzed if required.
+
+Two new attributes are present. The template-resource attribute contains a list
+of jobs (which must be resource jobs) that instantiate concrete job definitions
+from this template. The template-filter attribute applies filtering, to select
+a subset of records from each resource. As mentioned earlier, exactly one
+resource is allowed at this time. In the future we _may_ offer two or more but
+with additional join semantics that seems unnecessary at this stage.
+
+    template-resource: device
+    template-filter: device.category == "CDROM"
+    plugin: user-interact-verify
+    name: optical/read_{device.full_path}
+    user: root
+    command: optical_read_test {device.dev_path}
+    description: (removed)
+
+The previous definition used some additional shell-outs to compute the
+pathnames of block devices. This feature is now removed, leading to
+easier-to-understand code. The device local job would need to be amended to
+generate all required attributes.
+
+Impact
+======
+
+PlainBox would need to understand templates. That includes additional job for
+job validator, dependency resolver / test planner, etc. The new API would need
+to include an explicit transformation from a set of known definitions into a
+set of actual instantiated jobs. Applications would need to use this API
+similarly to how they currently handle "early jobs" to run all of the local
+jobs, but with less required error handling.
+
+A set of additional commands would be added to help developers how their
+template jobs transform into actual instances. This is an optional but, in my
+opinion, useful addition. It would allow developers to understand why something
+fails (how their template definition is expanded into an actual job
+definition).
+
+Local jobs would be unchanged and would remain supported as they are now.
+
+Future Work
+===========
+
+As we convert local jobs to template jobs, we could add an additional
+requirement that local jobs may only generate existing jobs, to implement the
+illy-defined "category" system, before that is deprecated and replaced.
+
+Eventually we could bump job definition version to 2, making local jobs
+unsupported. This will allow applications to drop complexity related such jobs.
+This can happen at any time in the future though.

=== added file 'cep/CEP-2.txt'
--- cep/CEP-2.txt	1970-01-01 00:00:00 +0000
+++ cep/CEP-2.txt	2014-12-19 12:11:52 +0000
@@ -0,0 +1,111 @@
+=============================
+Checkbox Enhancement Proposal
+=============================
+
+This proposal describes the new release process for internal Canonical
+customers of Checkbox. 
+
+Advantages
+==========
+
+* There is a tag associated with every release
+* We can eventually generate source tarballs
+* Tags from multiple components stored in lp:checkbox don't collide
+* There is a way to iterate on release candidates if important issues are found
+* All changes made in the release branch (including tags) end up in trunk
+* The current CI system is involved to catch mistakes
+
+New Process
+===========
+
+The new process is expressed as a series of shell commands and assumptions.
+
+Assumptions
+-----------
+
+* We have lp:checkbox as ~/trunk and lp:checkbox/release as ~/release
+* The process was followed before. This implies that lp:checkbox/release has
+  no patches that are not already in lp:checkbox
+* A number of new commands (not implemented) are available in the support
+  directory as well as sub-commands of setup.py. Those are easy to emulate
+  manually but should be implemented to fully automate the process.
+
+Cutting the release
+-------------------
+
+Off-line steps, you can do them as many times as you like:
+
+    ~$ bzr push -d trunk release
+    # TODO: pick version of packaging branck to go along with this
+    # do a test build, iterate if it fails
+    ~$ cd release
+    ~/release$ ./checkbox-old/setup.py bump-version-status --to=rc1
+    ~/release$ bzr tag $(./checkbox-old/setup.py --name)-v$(./checkbox-old/setup.py --version)
+
+On-line steps:
+
+    ~/release$ bzr push lp:checkbox/release
+    ~/release$ ./support/trigger-builds-in ppa:~checkbox-dev/testing
+
+Alternatively, instead of bzr push, propose the merge to launchpad,
+auto-approve it and have tarmac do test package builds using the packaging
+branch reference in support/packaging-revision
+
+Fixing issues found in release candidates
+-----------------------------------------
+
+Off-line steps::
+
+    ~/release$ while ./tree-broken; do
+    > (cd bzr && vim .)
+    > # Hack, cherry pick from trunk, fix locally, whatever
+    > (cd release && bzr commit)
+    > done
+    ~/release$ ./checkbox-old/setup.py bump-version-status --to=next-rc
+    ~/release$ bzr tag $(./checkbox-old/setup.py --name)-v$(./checkbox-old/setup.py --version)
+
+On-line steps::
+
+    ~/release$ bzr push lp:checkbox/release
+    ~/release$ ./support/trigger-builds-in ppa:~checkbox-dev/testing
+    ~/release$ ./support/send-email --to=... --topic "Checkboc Release Candidate Available" <<EOM
+    > The release candidate for the next checkbox release is available for
+    > testing, please check and file bugs ... and target them to milestone ...
+    > 
+    > And we should improve this message one day
+    > EOM
+
+Alternatively, instead of bzr push, propose the merge to launchpad,
+auto-approve it and have tarmac do test package builds using the packaging
+branch reference in support/packaging-revision
+
+Finalizing the release
+----------------------
+
+Off-line steps::
+
+    ~/release$ ./checkbox-old/setup.py bump-version-status --to=final
+    ~/release$ bzr tag $(./checkbox-old/setup.py --name)-v$(./checkbox-old/setup.py --version)
+
+On-line steps::
+
+    ~/release$ bzr push lp:checkbox/release
+    ~/release$ ./support/trigger-builds-in ppa:~checkbox-dev/testing
+    ~/release$ ./support/ppa-copy \
+    >   --from=ppa:~checkbox-dev/testing \
+    >   --to=ppa:~checkbox-dev/stable \
+    >   --packages=...
+    ~/release$ ./support/send-email --to=... --topic "Next checkbox release" <<EOM
+    > This is the next checkbox release.\
+    > It has been published to the stable PPA.
+    > 
+    > And we should really improve this message one day
+    > EOM
+    ~/release$ bzr lp-propose-merge lp:checkbox -m "post-release merge back to trunk" --approve
+
+Impact
+======
+
+If the new process is implemented correctly impact to our customers should be
+minimal. We need to communicate the purpose of the release candidate releases
+but apart from that the new process mirrors the effect of our current process

=== added file 'cep/CEP-3.txt'
--- cep/CEP-3.txt	1970-01-01 00:00:00 +0000
+++ cep/CEP-3.txt	2014-12-19 12:11:52 +0000
@@ -0,0 +1,34 @@
+
+Rationale
+=========
+
+Current efforts to make plainbox, checkbox and various job providers
+translatable are affected by the fact that we abuse JobDefinition.name as both
+english-like constant identifier and user-visible name. I would like to
+decouple those roles.
+
+Proposal
+========
+
+1. I would like to propose that the on-disk format field ``name`` be deprecated
+   and not used anymore.
+2. If the field ``name`` is used, it will be transparently mapped to ``id`` and
+   used to define the new job attribute ``JobDefinition.id``. All uses of
+   ``name`` will cause warnings to be logged. After a while we can remove that
+   feature and rejects the ``name`` field (for another period, after which we
+   could reuse it).
+3. A new field ``summary`` is defined and mandatory for all jobs. The summary
+   field must be one line, short (capped to some reasonable amount) and should
+   be derived from the current name field in a meaningful way (manually, one
+   time transition process).
+4. The ``summary`` field will be translatable
+5. PlainBox will resort to ``JobDefinition.summary`` instead of ``.id``
+   whenever the job needs to be displayed or converted to a string.
+
+Impact
+======
+
+This proposal would need to see forked job definitions *or* add support for
+``summary`` in the old checkbox. The level of required support is minimal, only
+to the point where current functionality does not regress
+

=== added file 'cep/CEP-4.txt'
--- cep/CEP-4.txt	1970-01-01 00:00:00 +0000
+++ cep/CEP-4.txt	2014-12-19 12:11:52 +0000
@@ -0,0 +1,133 @@
+======================
+PlainBox Target Device
+======================
+
+This proposal describes a method for executing jobs remotely, on one device
+(target), while running python3 and all of plainbox, on another device (host)
+
+Rationale
+=========
+
+During the development of first external PlainBox provider for Ubuntu Touch, we
+found that shipping python3, plainbox and a lot of data onto the phone and
+table target is challenging. Not only is the root filesystem read only,
+traditional package management is not available outside of non-standard
+builds. For the purpose of testing systems as similar to production systems as
+possible, making the root filesystem writable was rejected as an option.
+Another possibility, that was initially pursued, was to wrap everything related
+to plainbox and a set of providers into a big click package. While that
+approach worked it was too not without challenges. In the end the process was
+deemed to heavyweight and that idea was abandoned. This lead to the idea of
+running plainbox locally, one a typical Ubuntu environment and using adb or ssh
+to connect to the remote test target.
+
+An experimental provider, using plainbox 0.4, used adb push and adb shell, to
+copy the data and execute commands remotely. This approach was successful. This
+proposal aims to implement that functionality across the framework, taking it
+into account where appropriate.
+
+Changes
+=======
+
+Unless otherwise noted, everything remains local. This specifically includes
+session storage, which holds the bulk of the filesystem visible artifacts of a
+running session. This keeps the existing workflow, result retrieval and tooling
+identical in the remote and local use-cases.
+
+PLAINBOX_SESSION_SHARE
+----------------------
+
+The primary difference is that PLAINBOX_SESSION_SHARE would no longer be
+something that you can access locally. This is a temporary, per-session
+directory where jobs can exchange data of any kind. This change would allow all
+jobs to run the same, so no jobs would need to be modified for this setup to
+work and all the existing local jobs would run remotely, producing the same
+results.
+
+(This is also known as CHECKBOX_DATA but this name is deprecated)
+
+Execution controllers
+---------------------
+
+Execution controllers that prepare the environment for each job would
+necessarily change. Existing set of controllers can run jobs as the regular
+user, as a root user via sudo or as a root user via
+plainbox-trusted-launcher-1.
+
+In addition to user handing, those controllers handle the task of configuring
+the filesystem for a specific job. The new remote controller would need to
+ensure that provider data associated with the job that is to be executed is
+copied (using rsync, adb push or other similar command).
+
+Handling of root jobs would vary, depending on the capabilities of the remote
+target. For the first version of this proposal we don't have sufficient data or
+experience to draw conclusions as to how that should work exactly. The existing
+mechanisms that allow execution controllers to bid on the execution of a
+particular job is sufficient for specialized solutions to be accommodated,
+though.
+
+The act of running a job would need minor changes, specifically to transition
+all of the required environment across to the target device. Existing sudo and
+plainbox-trusted-launcher-1 execution controllers already offer that
+functionality. Actual new execution controllers (here, presumably for ssh and
+adb) would just prefix the command with the appropriate connection string and
+rely on system-level configuration for making a successful connection. This
+approach is flexible and could allow us to connect to a variety of targets,
+including phones and tables (using adb), remote servers in the cloud (using
+ssh) or development boards (using serial connections).
+
+Session State Meta Data
+-----------------------
+
+Session state meta data should be expanded to accurately store connection
+information as well as location of the temporary session storage directory.
+
+This will necessitate another version increase in the on-disk persistent
+storage used by plainbox but the necessary infrastructure, both for code and
+testing, was developed a while ago, while we introduced the app_data field.
+
+Command Line Interface
+----------------------
+
+PlainBox would grow a number of new command line arguments and options. The
+precise set is not clear yet, and largely depends on how execution targets are
+implemented. At the very least we'd like to provider a --target string, that
+uses a pseudo-URL syntax for defining the target connection string and the
+scheme to pick the target controller.
+
+A simple session might look like:
+
+    $ plainbox --target adb:some-serial-number run -i stub/true
+
+Unresolved Issues
+-----------------
+
+The following issues are not resolved as of this time. We believe that by
+implementing this proposal we will gain critical understanding and insight that
+may allow us to explore those problems and come up with further enhancement.
+
+1. Handling jobs that terminate host connection.
+
+   Certain jobs may terminate the host connection by issuing reboots, suspends,
+   or performing purposeful network reconfiguration. Such jobs will likely
+   cause the execution controller to time out and abort the connection.
+
+2. Handling sandboxing and general remote user constraints.
+
+   Remote execution targets may differ widely, compared to our traditional
+   environments. It is uncertain if our current abstractions are sufficient for
+   handling user identity changes and general privilege elevation tasks.
+
+3. Local and remote interaction.
+
+   Implementation of this proposal will open up interesting possibilities,
+   including interactions of jobs executed locally and remotely within a single
+   session.
+
+   This task is mentioned but not explored, as we are unsure about the exact
+   requirements as this time. Such interactions may include:
+
+   * having jobs that execute locally to verify presence, access and
+     configuration of a remote target
+   * having jobs that provision and tear down the remote target
+   * having multiple remote targets cooperate during a single execution session

